@@ -1,13 +1,14 @@
 use std::path::PathBuf;
 
-use image_hasher::{FilterType, HashAlg};
-
-use czkawka_core::common_dir_traversal::CheckingMethod;
-use czkawka_core::common_tool::DeleteMethod;
-use czkawka_core::duplicate::HashType;
-use czkawka_core::same_music::MusicSimilarity;
-use czkawka_core::similar_images::SimilarityPreset;
 use czkawka_core::CZKAWKA_VERSION;
+use czkawka_core::common::model::{CheckingMethod, HashType};
+use czkawka_core::common::tool_data::DeleteMethod;
+use czkawka_core::tools::broken_files::CheckedTypes;
+use czkawka_core::tools::same_music::MusicSimilarity;
+use czkawka_core::tools::similar_images::SimilarityPreset;
+use czkawka_core::tools::similar_videos::{ALLOWED_SKIP_FORWARD_AMOUNT, ALLOWED_VID_HASH_DURATION, DEFAULT_SKIP_FORWARD_AMOUNT, crop_detect_from_str_opt};
+use image_hasher::{FilterType, HashAlg};
+use vid_dup_finder_lib::Cropdetect;
 
 #[derive(clap::Parser)]
 #[clap(
@@ -72,22 +73,38 @@ pub enum Commands {
         after_help = "EXAMPLE:\n    czkawka broken -d /home/kicikici/ /home/szczek -e /home/kicikici/jestempsem -x jpg -f results.txt"
     )]
     BrokenFiles(BrokenFilesArgs),
-    #[clap(name = "video", about = "Finds similar video files", after_help = "EXAMPLE:\n    czkawka videos -d /home/rafal -f results.txt")]
+    #[clap(name = "video", about = "Finds similar video files", after_help = "EXAMPLE:\n    czkawka video -d /home/rafal -f results.txt")]
     SimilarVideos(SimilarVideosArgs),
     #[clap(
         name = "ext",
         about = "Finds files with invalid extensions",
-        after_help = "EXAMPLE:\n    czkawka broken -d /home/czokolada/ -f results.txt"
+        after_help = "EXAMPLE:\n    czkawka ext -d /home/czokolada/ -f results.txt"
     )]
     BadExtensions(BadExtensionsArgs),
-    #[clap(name = "tester", about = "Small utility to test supported speed of ", after_help = "EXAMPLE:\n    czkawka tester")]
-    Tester,
 }
 
 #[derive(Debug, clap::Args)]
 pub struct DuplicatesArgs {
     #[clap(flatten)]
     pub common_cli_items: CommonCliItems,
+    #[clap(flatten)]
+    pub reference_directories: ReferenceDirectories,
+    #[clap(
+        short = 'Z',
+        long,
+        value_parser = parse_minimal_file_size,
+        default_value = "257144",
+        help = "Minimum prehash cache file size in bytes",
+        long_help = "Minimum size of prehash cached files in bytes"
+    )]
+    pub minimal_prehash_cache_file_size: u64,
+    #[clap(
+        short = 'u',
+        long,
+        help = "Use prehash cache",
+        long_help = "Use prehash cache to speed up the scanning process by avoiding rehashing files that have already been hashed"
+    )]
+    pub use_prehash_cache: bool,
     #[clap(
         short,
         long,
@@ -182,6 +199,8 @@ pub struct TemporaryArgs {
 pub struct SimilarImagesArgs {
     #[clap(flatten)]
     pub common_cli_items: CommonCliItems,
+    #[clap(flatten)]
+    pub reference_directories: ReferenceDirectories,
     #[clap(
         short,
         long,
@@ -205,7 +224,7 @@ pub struct SimilarImagesArgs {
         long,
         default_value = "High",
         value_parser = parse_similar_images_similarity,
-        help = "Similairty level (Minimal, VerySmall, Small, Medium, High, VeryHigh, Original)",
+        help = "Similarity level (Minimal, VerySmall, Small, Medium, High, VeryHigh, Original)",
         long_help = "Methods to choose similarity level of images which will be considered as duplicated."
     )]
     pub similarity_preset: SimilarityPreset,
@@ -215,12 +234,14 @@ pub struct SimilarImagesArgs {
     pub allow_hard_links: AllowHardLinks,
     #[clap(flatten)]
     pub dry_run: DryRun,
+    #[clap(flatten)]
+    pub ignore_same_size: IgnoreSameSize,
     #[clap(
         short = 'g',
         long,
         default_value = "Gradient",
         value_parser = parse_similar_hash_algorithm,
-        help = "Hash algorithm (allowed: Mean, Gradient, Blockhash, VertGradient, DoubleGradient)"
+        help = "Hash algorithm (allowed: Mean, Gradient, Blockhash, VertGradient, DoubleGradient, Median)"
     )]
     pub hash_alg: HashAlg,
     #[clap(
@@ -246,16 +267,22 @@ pub struct SameMusicArgs {
     #[clap(flatten)]
     pub common_cli_items: CommonCliItems,
     #[clap(flatten)]
+    pub reference_directories: ReferenceDirectories,
+    #[clap(flatten)]
     pub delete_method: DMethod,
     #[clap(flatten)]
     pub dry_run: DryRun,
+    #[clap(short, long, help = "Approximate comparison of music tags.")]
+    pub approximate_comparison: bool,
+    #[clap(short, long, help = "Compare fingerprints only with similar titles.")]
+    pub compare_fingerprints_only_with_similar_titles: bool,
     #[clap(
         short = 'z',
         long,
         default_value = "track_title,track_artist",
         value_parser = parse_music_duplicate_type,
-        help = "Search method (track_title,track_artist,year,bitrate,genre,length))",
-        long_help = "Sets which rows must be equal to set this files as duplicates(may be mixed, but must be divided by commas)."
+        help = "Search method (track_title,track_artist,year,bitrate,genre,length)",
+        long_help = "Sets which rows must be equal to set these files as duplicates (may be mixed, but must be divided by commas)."
     )]
     pub music_similarity: MusicSimilarity,
     #[clap(
@@ -264,7 +291,7 @@ pub struct SameMusicArgs {
         default_value = "TAGS",
         value_parser = parse_checking_method_same_music,
         help = "Search method (CONTENT, TAGS)",
-        long_help = "Methods to search files.\nCONTENT - finds similar audio files by content, TAGS - finds similar images by tags, needs to set"
+        long_help = "Methods to search files.\nCONTENT - finds similar audio files by content, TAGS - finds similar music by tags."
     )]
     pub search_method: CheckingMethod,
     #[clap(
@@ -295,7 +322,7 @@ pub struct SameMusicArgs {
     )]
     pub minimum_segment_duration: f32,
     #[clap(
-        short = 'd',
+        short = 'Y',
         long,
         value_parser = parse_maximum_difference,
         default_value = "2.0",
@@ -348,6 +375,15 @@ pub struct BrokenFilesArgs {
     pub common_cli_items: CommonCliItems,
     #[clap(short = 'D', long, help = "Delete found files")]
     pub delete_files: bool,
+    #[clap(
+        short,
+        long,
+        default_value = "PDF",
+        value_parser = parse_broken_files,
+        help = "Checking file types (PDF, AUDIO, IMAGE, ARCHIVE)",
+        long_help = "Methods to search files - default PDF.\nPDF - finds broken PDF files,\nAUDIO - finds broken audio files,\nIMAGE - finds broken image files,\nARCHIVE - finds broken archive files"
+    )]
+    pub checked_types: Vec<CheckedTypes>,
 }
 
 #[derive(Debug, clap::Args)]
@@ -355,11 +391,15 @@ pub struct SimilarVideosArgs {
     #[clap(flatten)]
     pub common_cli_items: CommonCliItems,
     #[clap(flatten)]
+    pub reference_directories: ReferenceDirectories,
+    #[clap(flatten)]
     pub delete_method: DMethod,
     #[clap(flatten)]
     pub allow_hard_links: AllowHardLinks,
     #[clap(flatten)]
     pub dry_run: DryRun,
+    #[clap(flatten)]
+    pub ignore_same_size: IgnoreSameSize,
     #[clap(
         short,
         long,
@@ -387,6 +427,33 @@ pub struct SimilarVideosArgs {
         long_help = "Maximum difference between video frames, bigger value means that videos can looks more and more different (allowed values <0,20>)"
     )]
     pub tolerance: i32,
+    #[clap(
+        short = 'U',
+        long,
+        default_value_t = DEFAULT_SKIP_FORWARD_AMOUNT,
+        value_parser = parse_skip_forward_amount,
+        help = "Skip forward amount in seconds (allowed values: 0-300, default: 15)",
+        long_help = "Amount of seconds to skip forward in video. Allowed values are from 0 to 300. 0 means that no skipping will be done. Default is 15."
+    )]
+    pub skip_forward_amount: u32,
+    #[clap(
+        short = 'B',
+        long,
+        default_value = "letterbox",
+        value_parser = parse_crop_detect,
+        help = "Crop detect method (none, letterbox, motion)",
+        long_help = "Method to crop video frames",
+    )]
+    pub crop_detect: Cropdetect,
+    #[clap(
+        short = 'A',
+        long,
+        default_value = "10",
+        value_parser = parse_scan_duration,
+        help = "Scan duration in seconds",
+        long_help = "Duration of scanning video in seconds.",
+    )]
+    pub scan_duration: u32,
 }
 
 #[derive(Debug, clap::Args)]
@@ -404,7 +471,7 @@ pub struct CommonCliItems {
         long,
         required = true,
         help = "Directorie(s) to search",
-        long_help = "List of directorie(s) which will be searched(absolute path)"
+        long_help = "List of directorie(s) which will be searched(absolute path) - this directories are not set as reference folders"
     )]
     pub directories: Vec<PathBuf>,
     #[clap(
@@ -428,7 +495,7 @@ pub struct CommonCliItems {
         long_help = "List of checked files with provided extension(s). There are also helpful macros which allow to easy use a typical extensions like:\nIMAGE(\"jpg,kra,gif,png,bmp,tiff,hdr,svg\"),\nTEXT(\"txt,doc,docx,odt,rtf\"),\nVIDEO(\"mp4,flv,mkv,webm,vob,ogv,gifv,avi,mov,wmv,mpg,m4v,m4p,mpeg,3gp\") or\nMUSIC(\"mp3,flac,ogg,tta,wma,webm\")\n "
     )]
     pub allowed_extensions: Vec<String>,
-    #[clap(short = 'E', long, help = "Excluded file extension(s)", long_help = "List of extensions, that will be removed from search.\n ")]
+    #[clap(short = 'P', long, help = "Excluded file extension(s)", long_help = "List of extensions, that will be removed from search.\n ")]
     pub excluded_extensions: Vec<String>,
     #[clap(flatten)]
     pub file_to_save: FileToSave,
@@ -441,6 +508,20 @@ pub struct CommonCliItems {
     #[cfg(target_family = "unix")]
     #[clap(short = 'X', long, help = "Exclude files on other filesystems")]
     pub exclude_other_filesystems: bool,
+    #[clap(flatten)]
+    pub do_not_print: DoNotPrint,
+    #[clap(short = 'W', long, help = "Ignore error code when files are found")]
+    pub ignore_error_code_on_found: bool,
+    #[clap(short = 'H', long, help = "Disable cache")]
+    pub disable_cache: bool,
+}
+
+#[derive(Debug, clap::Args)]
+pub struct DoNotPrint {
+    #[clap(short = 'N', long, help = "Do not print the results to the console")]
+    pub do_not_print_results: bool,
+    #[clap(short = 'M', long, help = "Do not print info/warnings/errors from the program to console")]
+    pub do_not_print_messages: bool,
 }
 
 #[derive(Debug, clap::Args)]
@@ -450,8 +531,8 @@ pub struct DMethod {
         long,
         default_value = "NONE",
         value_parser = parse_delete_method,
-        help = "Delete method (AEN, AEO, ON, OO, HARD)",
-        long_help = "Methods to delete the files.\nAEN - All files except the newest,\nAEO - All files except the oldest,\nON - Only 1 file, the newest,\nOO - Only 1 file, the oldest\nHARD - create hard link\nNONE - not delete files"
+        help = "Delete method (AEN, AEO, ON, OO, AEB, AES, OE, OS, HARD)",
+        long_help = "Methods to delete the files.\nAEN - All files except the newest,\nAEO - All files except the oldest,\nON - Only 1 file, the newest,\nOO - Only 1 file, the oldest\nAEB - All files except the biggest,\nAES - All files except the smallest,\nOB - Only 1 file, the biggest,\nOS - Only 1 file, the smallest\nHARD - create hard link\nNONE - not delete files"
     )]
     pub delete_method: DeleteMethod,
 }
@@ -462,6 +543,17 @@ pub struct FileToSave {
     pub file_to_save: Option<PathBuf>,
 }
 
+#[derive(Debug, clap::Args)]
+pub struct ReferenceDirectories {
+    #[clap(
+        short,
+        long,
+        help = "Reference directorie(s) to search",
+        long_help = "List of directorie(s) which will be searched(absolute path) - this directories are set as reference folders, so will not be visible in the results"
+    )]
+    pub reference_directories: Vec<PathBuf>,
+}
+
 #[derive(Debug, clap::Args)]
 pub struct JsonCompactFileToSave {
     #[clap(short = 'C', long, value_name = "json-file-name", help = "Saves the results into the compact json file")]
@@ -492,8 +584,14 @@ pub struct DryRun {
     pub dry_run: bool,
 }
 
+#[derive(Debug, clap::Args)]
+pub struct IgnoreSameSize {
+    #[clap(short = 'J', long, help = "Ignore files with the same size, leaving only one file of each size")]
+    pub ignore_same_size: bool,
+}
+
 impl FileToSave {
-    pub fn file_name(&self) -> Option<&str> {
+    pub(crate) fn file_name(&self) -> Option<&str> {
         if let Some(file_name) = &self.file_to_save {
             return file_name.to_str();
         }
@@ -502,7 +600,7 @@ impl FileToSave {
     }
 }
 impl JsonCompactFileToSave {
-    pub fn file_name(&self) -> Option<&str> {
+    pub(crate) fn file_name(&self) -> Option<&str> {
         if let Some(file_name) = &self.compact_file_to_save {
             return file_name.to_str();
         }
@@ -511,7 +609,7 @@ impl JsonCompactFileToSave {
     }
 }
 impl JsonPrettyFileToSave {
-    pub fn file_name(&self) -> Option<&str> {
+    pub(crate) fn file_name(&self) -> Option<&str> {
         if let Some(file_name) = &self.pretty_file_to_save {
             return file_name.to_str();
         }
@@ -520,6 +618,38 @@ impl JsonPrettyFileToSave {
     }
 }
 
+fn parse_scan_duration(s: &str) -> Result<u32, String> {
+    match s.parse::<u32>() {
+        Ok(scan_duration) => {
+            if ALLOWED_VID_HASH_DURATION.contains(&scan_duration) {
+                Ok(scan_duration)
+            } else {
+                Err(format!("Scan duration must be one of: {ALLOWED_VID_HASH_DURATION:?}"))
+            }
+        }
+        Err(e) => Err(e.to_string()),
+    }
+}
+
+fn parse_crop_detect(src: &str) -> Result<Cropdetect, String> {
+    match crop_detect_from_str_opt(src) {
+        Some(crop_detect) => Ok(crop_detect),
+        None => Err(format!("Crop detect \"{src}\" is not valid")),
+    }
+}
+
+fn parse_skip_forward_amount(src: &str) -> Result<u32, String> {
+    match src.parse::<u32>() {
+        Ok(skip_forward_amount) => {
+            if !ALLOWED_SKIP_FORWARD_AMOUNT.contains(&skip_forward_amount) {
+                Err(format!("Skip forward amount must be one of: {ALLOWED_SKIP_FORWARD_AMOUNT:?}"))
+            } else {
+                Ok(skip_forward_amount)
+            }
+        }
+        Err(e) => Err(e.to_string()),
+    }
+}
 fn parse_hash_type(src: &str) -> Result<HashType, &'static str> {
     match src.to_ascii_lowercase().as_str() {
         "blake3" => Ok(HashType::Blake3),
@@ -552,11 +682,21 @@ fn parse_checking_method_duplicate(src: &str) -> Result<CheckingMethod, &'static
     }
 }
 
+fn parse_broken_files(src: &str) -> Result<CheckedTypes, &'static str> {
+    match src.to_ascii_lowercase().as_str() {
+        "pdf" => Ok(CheckedTypes::PDF),
+        "audio" => Ok(CheckedTypes::AUDIO),
+        "image" => Ok(CheckedTypes::IMAGE),
+        "archive" => Ok(CheckedTypes::ARCHIVE),
+        _ => Err("Couldn't parse the broken files type (allowed: PDF, AUDIO, IMAGE, ARCHIVE)"),
+    }
+}
+
 fn parse_checking_method_same_music(src: &str) -> Result<CheckingMethod, &'static str> {
     match src.to_ascii_lowercase().as_str() {
         "tags" => Ok(CheckingMethod::AudioTags),
         "content" => Ok(CheckingMethod::AudioContent),
-        _ => Err("Couldn't parse the searc method (allowed: TAGS, CONTENT)"),
+        _ => Err("Couldn't parse the search method (allowed: TAGS, CONTENT)"),
     }
 }
 
@@ -568,7 +708,11 @@ fn parse_delete_method(src: &str) -> Result<DeleteMethod, &'static str> {
         "hard" => Ok(DeleteMethod::HardLink),
         "on" => Ok(DeleteMethod::OneNewest),
         "oo" => Ok(DeleteMethod::OneOldest),
-        _ => Err("Couldn't parse the delete method (allowed: AEN, AEO, ON, OO, HARD)"),
+        "aeb" => Ok(DeleteMethod::AllExceptBiggest),
+        "aes" => Ok(DeleteMethod::AllExceptSmallest),
+        "ob" => Ok(DeleteMethod::OneBiggest),
+        "os" => Ok(DeleteMethod::OneSmallest),
+        _ => Err("Couldn't parse the delete method (allowed: AEN, AEO, ON, OO, HARD, AEB, AES, OB, OS)"),
     }
 }
 
@@ -580,7 +724,8 @@ fn parse_similar_images_similarity(src: &str) -> Result<SimilarityPreset, &'stat
         "medium" => Ok(SimilarityPreset::Medium),
         "high" => Ok(SimilarityPreset::High),
         "veryhigh" => Ok(SimilarityPreset::VeryHigh),
-        _ => Err("Couldn't parse the image similarity preset (allowed: Minimal, VerySmall, Small, Medium, High, VeryHigh)"),
+        "original" => Ok(SimilarityPreset::Original),
+        _ => Err("Couldn't parse the image similarity preset (allowed: Minimal, VerySmall, Small, Medium, High, VeryHigh, Original)"),
     }
 }
 
@@ -609,9 +754,9 @@ fn parse_similar_image_filter(src: &str) -> Result<FilterType, String> {
         "lanczos3" => FilterType::Lanczos3,
         "nearest" => FilterType::Nearest,
         "triangle" => FilterType::Triangle,
-        "faussian" => FilterType::Gaussian,
+        "gaussian" => FilterType::Gaussian,
         "catmullrom" => FilterType::CatmullRom,
-        _ => return Err("Couldn't parse the image resize filter (allowed: Lanczos3, Nearest, Triangle, Faussian, Catmullrom)".to_string()),
+        _ => return Err("Couldn't parse the image resize filter (allowed: Lanczos3, Nearest, Triangle, Gaussian, Catmullrom)".to_string()),
     };
     Ok(filter_type)
 }
@@ -623,7 +768,8 @@ fn parse_similar_hash_algorithm(src: &str) -> Result<HashAlg, String> {
         "blockhash" => HashAlg::Blockhash,
         "vertgradient" => HashAlg::VertGradient,
         "doublegradient" => HashAlg::DoubleGradient,
-        _ => return Err("Couldn't parse the hash algorithm (allowed: Mean, Gradient, Blockhash, VertGradient, DoubleGradient)".to_string()),
+        "median" => HashAlg::Median,
+        _ => return Err("Couldn't parse the hash algorithm (allowed: Mean, Gradient, Blockhash, VertGradient, DoubleGradient, Median)".to_string()),
     };
     Ok(algorithm)
 }
@@ -686,7 +832,7 @@ OPTIONS:
 SUBCOMMANDS:
 {subcommands}
 
-    try "{usage} -h" to get more info about a specific tool
+    try "{usage} <COMMAND> -h" to get more info about a specific tool
 
 EXAMPLES:
     {bin} dup -d /home/rafal -e /home/rafal/Obrazy  -m 25 -x 7z rar IMAGE -s hash -f results.txt -D aeo
@@ -695,7 +841,7 @@ EXAMPLES:
     {bin} empty-files -d /home/rafal /home/szczekacz -e /home/rafal/Pulpit -R -f results.txt
     {bin} temp -d /home/rafal/ -E */.git */tmp* *Pulpit -f results.txt -D
     {bin} image -d /home/rafal -e /home/rafal/Pulpit -f results.txt
-    {bin} music -d /home/rafal -e /home/rafal/Pulpit -z "artist,year, ARTISTALBUM, ALBUM___tiTlE"  -f results.txt
+    {bin} music -d /home/rafal -e /home/rafal/Pulpit -z \"artist,year,ARTISTALBUM,ALBUM___tiTlE\"  -f results.txt
     {bin} symlinks -d /home/kicikici/ /home/szczek -e /home/kicikici/jestempsem -x jpg -f results.txt
     {bin} broken -d /home/mikrut/ -e /home/mikrut/trakt -f results.txt
-    {bin} extnp -d /home/mikrut/ -e /home/mikrut/trakt -f results.txt"#;
+    {bin} ext -d /home/mikrut/ -e /home/mikrut/trakt -f results.txt"#;
