@@ -1,26 +1,24 @@
 use std::cell::RefCell;
 use std::rc::Rc;
 
+use czkawka_core::common::image::get_dynamic_image_from_path;
 use gdk4::gdk_pixbuf::{InterpType, Pixbuf};
 use gtk4::prelude::*;
 use gtk4::{Align, CheckButton, Image, ListStore, Orientation, ScrolledWindow, TreeIter, TreeModel, TreePath, TreeSelection, Widget};
 use image::DynamicImage;
-
-#[cfg(feature = "heif")]
-use czkawka_core::common::get_dynamic_image_from_heic;
-use czkawka_core::common::HEIC_EXTENSIONS;
+use log::error;
 
 use crate::flg;
 use crate::gui_structs::gui_data::GuiData;
 use crate::help_functions::{
     count_number_of_groups, get_all_direct_children, get_full_name_from_path_name, get_max_file_name, get_pixbuf_from_dynamic_image, resize_pixbuf_dimension,
 };
-use crate::notebook_info::{NotebookObject, NOTEBOOKS_INFO};
+use crate::notebook_info::{NOTEBOOKS_INFO, NotebookObject};
 
 const BIG_PREVIEW_SIZE: i32 = 600;
 const SMALL_PREVIEW_SIZE: i32 = 130;
 
-pub fn connect_button_compare(gui_data: &GuiData) {
+pub(crate) fn connect_button_compare(gui_data: &GuiData) {
     let button_compare = gui_data.bottom_buttons.buttons_compare.clone();
     let window_compare = gui_data.compare_images.window_compare.clone();
     let notebook_main = gui_data.main_notebook.notebook_main.clone();
@@ -44,22 +42,24 @@ pub fn connect_button_compare(gui_data: &GuiData) {
     let image_compare_left = gui_data.compare_images.image_compare_left.clone();
     let image_compare_right = gui_data.compare_images.image_compare_right.clone();
 
+    let check_button_settings_use_rust_preview = gui_data.settings.check_button_settings_use_rust_preview.clone();
+
     window_compare.set_default_size(700, 700);
 
     button_compare.connect_clicked(move |_| {
-        let nb_number = notebook_main.current_page().unwrap();
+        let nb_number = notebook_main.current_page().expect("Current page not set");
         let tree_view = &main_tree_views[nb_number as usize];
         let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().unwrap();
+        let model = tree_view.model().expect("Missing tree_view model");
 
-        let group_number = count_number_of_groups(tree_view, nb_object.column_header.unwrap());
+        let group_number = count_number_of_groups(tree_view, nb_object.column_header.expect("Missing column_header"));
 
         if group_number == 0 {
             return;
         }
 
         // Check selected items
-        let (current_group, tree_path) = get_current_group_and_iter_from_selection(&model, &tree_view.selection(), nb_object.column_header.unwrap());
+        let (current_group, tree_path) = get_current_group_and_iter_from_selection(&model, &tree_view.selection(), nb_object.column_header.expect("Missing column_header"));
 
         *shared_current_of_groups.borrow_mut() = current_group;
         *shared_numbers_of_groups.borrow_mut() = group_number;
@@ -81,6 +81,7 @@ pub fn connect_button_compare(gui_data: &GuiData) {
             &shared_using_for_preview,
             &button_go_previous_compare_group,
             &button_go_next_compare_group,
+            &check_button_settings_use_rust_preview,
         );
 
         window_compare.show();
@@ -125,18 +126,25 @@ pub fn connect_button_compare(gui_data: &GuiData) {
     let image_compare_left = gui_data.compare_images.image_compare_left.clone();
     let image_compare_right = gui_data.compare_images.image_compare_right.clone();
 
+    let check_button_settings_use_rust_preview = gui_data.settings.check_button_settings_use_rust_preview.clone();
+
     button_go_previous_compare_group.connect_clicked(move |button_go_previous_compare_group| {
-        let nb_number = notebook_main.current_page().unwrap();
+        let nb_number = notebook_main.current_page().expect("Current page not set");
         let tree_view = &main_tree_views[nb_number as usize];
         let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().unwrap();
+        let model = tree_view.model().expect("Missing tree_view model");
 
         *shared_current_of_groups.borrow_mut() -= 1;
 
         let current_group = *shared_current_of_groups.borrow();
         let group_number = *shared_numbers_of_groups.borrow();
 
-        let tree_path = move_iter(&model, shared_current_path.borrow().as_ref().unwrap(), nb_object.column_header.unwrap(), false);
+        let tree_path = move_iter(
+            &model,
+            shared_current_path.borrow().as_ref().expect("Missing current path"),
+            nb_object.column_header.expect("Missing column_header"),
+            false,
+        );
 
         populate_groups_at_start(
             nb_object,
@@ -155,6 +163,7 @@ pub fn connect_button_compare(gui_data: &GuiData) {
             &shared_using_for_preview,
             button_go_previous_compare_group,
             &button_go_next_compare_group,
+            &check_button_settings_use_rust_preview,
         );
     });
 
@@ -177,18 +186,25 @@ pub fn connect_button_compare(gui_data: &GuiData) {
     let image_compare_left = gui_data.compare_images.image_compare_left.clone();
     let image_compare_right = gui_data.compare_images.image_compare_right.clone();
 
+    let check_button_settings_use_rust_preview = gui_data.settings.check_button_settings_use_rust_preview.clone();
+
     button_go_next_compare_group.connect_clicked(move |button_go_next_compare_group| {
-        let nb_number = notebook_main.current_page().unwrap();
+        let nb_number = notebook_main.current_page().expect("Current page not set");
         let tree_view = &main_tree_views[nb_number as usize];
         let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().unwrap();
+        let model = tree_view.model().expect("Missing tree_view model");
 
         *shared_current_of_groups.borrow_mut() += 1;
 
         let current_group = *shared_current_of_groups.borrow();
         let group_number = *shared_numbers_of_groups.borrow();
 
-        let tree_path = move_iter(&model, shared_current_path.borrow().as_ref().unwrap(), nb_object.column_header.unwrap(), true);
+        let tree_path = move_iter(
+            &model,
+            shared_current_path.borrow().as_ref().expect("Missing current path"),
+            nb_object.column_header.expect("Missing column_header"),
+            true,
+        );
 
         populate_groups_at_start(
             nb_object,
@@ -207,6 +223,7 @@ pub fn connect_button_compare(gui_data: &GuiData) {
             &shared_using_for_preview,
             &button_go_previous_compare_group,
             button_go_next_compare_group,
+            &check_button_settings_use_rust_preview,
         );
     });
 
@@ -216,20 +233,28 @@ pub fn connect_button_compare(gui_data: &GuiData) {
     let shared_current_path = gui_data.compare_images.shared_current_path.clone();
     let main_tree_views = gui_data.main_notebook.get_main_tree_views();
     check_button_left_preview_text.connect_toggled(move |check_button_left_preview_text| {
-        let nb_number = notebook_main.current_page().unwrap();
+        let nb_number = notebook_main.current_page().expect("Current page not set");
         let tree_view = &main_tree_views[nb_number as usize];
         let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().unwrap().downcast::<ListStore>().unwrap();
+        let model = tree_view
+            .model()
+            .expect("Missing tree_view model")
+            .downcast::<ListStore>()
+            .expect("Failed to downcast to ListStore");
 
-        let main_tree_path = shared_current_path.borrow().as_ref().unwrap().clone();
-        let this_tree_path = shared_using_for_preview.borrow().0.clone().unwrap();
+        let main_tree_path = shared_current_path.borrow().as_ref().expect("Missing current path").clone();
+        let this_tree_path = shared_using_for_preview.borrow().0.clone().expect("Missing left preview path");
         if main_tree_path == this_tree_path {
             return; // Selected header, so we don't need to select result in treeview
-                    // TODO this should be handled by disabling entirely check box
+            // TODO this should be handled by disabling entirely check box
         }
 
         let is_active = check_button_left_preview_text.is_active();
-        model.set_value(&model.iter(&this_tree_path).unwrap(), nb_object.column_selection as u32, &is_active.to_value());
+        model.set_value(
+            &model.iter(&this_tree_path).expect("Using invalid tree_path"),
+            nb_object.column_selection as u32,
+            &is_active.to_value(),
+        );
     });
 
     let check_button_right_preview_text = gui_data.compare_images.check_button_right_preview_text.clone();
@@ -239,20 +264,28 @@ pub fn connect_button_compare(gui_data: &GuiData) {
     let main_tree_views = gui_data.main_notebook.get_main_tree_views();
 
     check_button_right_preview_text.connect_toggled(move |check_button_right_preview_text| {
-        let nb_number = notebook_main.current_page().unwrap();
+        let nb_number = notebook_main.current_page().expect("Current page not set");
         let tree_view = &main_tree_views[nb_number as usize];
         let nb_object = &NOTEBOOKS_INFO[nb_number as usize];
-        let model = tree_view.model().unwrap().downcast::<ListStore>().unwrap();
+        let model = tree_view
+            .model()
+            .expect("Missing tree_view model")
+            .downcast::<ListStore>()
+            .expect("Failed to downcast to ListStore");
 
-        let main_tree_path = shared_current_path.borrow().as_ref().unwrap().clone();
-        let this_tree_path = shared_using_for_preview.borrow().1.clone().unwrap();
+        let main_tree_path = shared_current_path.borrow().as_ref().expect("Missing current path").clone();
+        let this_tree_path = shared_using_for_preview.borrow().1.clone().expect("Missing right preview path");
         if main_tree_path == this_tree_path {
             return; // Selected header, so we don't need to select result in treeview
-                    // TODO this should be handled by disabling entirely check box
+            // TODO this should be handled by disabling entirely check box
         }
 
         let is_active = check_button_right_preview_text.is_active();
-        model.set_value(&model.iter(&this_tree_path).unwrap(), nb_object.column_selection as u32, &is_active.to_value());
+        model.set_value(
+            &model.iter(&this_tree_path).expect("Using invalid tree_path"),
+            nb_object.column_selection as u32,
+            &is_active.to_value(),
+        );
     });
 }
 
@@ -273,6 +306,7 @@ fn populate_groups_at_start(
     shared_using_for_preview: &Rc<RefCell<(Option<TreePath>, Option<TreePath>)>>,
     button_go_previous_compare_group: &gtk4::Button,
     button_go_next_compare_group: &gtk4::Button,
+    check_button_settings_use_rust_preview: &CheckButton,
 ) {
     if current_group == 1 {
         button_go_previous_compare_group.set_sensitive(false);
@@ -285,10 +319,16 @@ fn populate_groups_at_start(
         button_go_next_compare_group.set_sensitive(true);
     }
 
-    let all_vec = get_all_path(model, &tree_path, nb_object.column_header.unwrap(), nb_object.column_path, nb_object.column_name);
+    let all_vec = get_all_path(
+        model,
+        &tree_path,
+        nb_object.column_header.expect("Missing column_header"),
+        nb_object.column_path,
+        nb_object.column_name,
+    );
     *shared_current_path.borrow_mut() = Some(tree_path);
 
-    let cache_all_images = generate_cache_for_results(all_vec);
+    let cache_all_images = generate_cache_for_results(all_vec, check_button_settings_use_rust_preview.is_active());
 
     // This is safe, because cache have at least 2 results
     image_compare_left.set_paintable(cache_all_images[0].2.paintable().as_ref());
@@ -325,9 +365,15 @@ fn populate_groups_at_start(
     *shared_image_cache.borrow_mut() = cache_all_images.clone();
 
     let mut found = false;
-    for i in get_all_direct_children(&scrolled_window_compare_choose_images.child().unwrap().downcast::<gtk4::Viewport>().unwrap()) {
+    for i in get_all_direct_children(
+        &scrolled_window_compare_choose_images
+            .child()
+            .expect("Failed to get child of scrolled_window_compare_choose_images")
+            .downcast::<gtk4::Viewport>()
+            .expect("Failed to downcast to Viewport"),
+    ) {
         if i.widget_name() == "all_box" {
-            let gtk_box = i.downcast::<gtk4::Box>().unwrap();
+            let gtk_box = i.downcast::<gtk4::Box>().expect("Failed to downcast to Box");
             update_bottom_buttons(&gtk_box, shared_using_for_preview, shared_image_cache);
             found = true;
             break;
@@ -335,13 +381,13 @@ fn populate_groups_at_start(
     }
     assert!(found);
 
-    let is_active = model.get::<bool>(&model.iter(&cache_all_images[0].4).unwrap(), nb_object.column_selection);
+    let is_active = model.get::<bool>(&model.iter(&cache_all_images[0].4).expect("Using invalid tree_path"), nb_object.column_selection);
     check_button_left_preview_text.set_active(is_active);
-    let is_active = model.get::<bool>(&model.iter(&cache_all_images[1].4).unwrap(), nb_object.column_selection);
+    let is_active = model.get::<bool>(&model.iter(&cache_all_images[1].4).expect("Using invalid tree_path"), nb_object.column_selection);
     check_button_right_preview_text.set_active(is_active);
 }
 
-fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>) -> Vec<(String, String, Image, Image, TreePath)> {
+fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>, use_rust_loader: bool) -> Vec<(String, String, Image, Image, TreePath)> {
     // TODO use here threads,
     // For now threads cannot be used because Image and TreeIter cannot be used in threads
     let mut cache_all_images = Vec::new();
@@ -349,44 +395,31 @@ fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>)
         let small_img = Image::new();
         let big_img = Image::new();
 
-        let mut pixbuf = get_pixbuf_from_dynamic_image(&DynamicImage::new_rgb8(1, 1)).unwrap();
-        let extension_lowercase = full_path.split('.').last().map(str::to_lowercase);
-        let is_heic = match extension_lowercase {
-            Some(extension) => HEIC_EXTENSIONS.iter().any(|e| e == &extension),
-            None => false,
-        };
+        let mut pixbuf = get_pixbuf_from_dynamic_image(&DynamicImage::new_rgb8(1, 1)).expect("Failed to create pixbuf");
 
-        if is_heic {
-            #[allow(clippy::never_loop)]
-            'czystka: loop {
-                #[cfg(feature = "heif")]
-                if is_heic {
-                    match get_dynamic_image_from_heic(&full_path) {
+        if use_rust_loader {
+            match get_dynamic_image_from_path(&full_path) {
+                Ok(t) => {
+                    match get_pixbuf_from_dynamic_image(&t) {
                         Ok(t) => {
-                            match get_pixbuf_from_dynamic_image(&t) {
-                                Ok(t) => {
-                                    pixbuf = t;
-                                }
-                                Err(e) => {
-                                    println!("Failed to open image {full_path}, reason {e}");
-                                }
-                            };
+                            pixbuf = t;
                         }
                         Err(e) => {
-                            println!("Failed to open image {full_path}, reason {e}");
+                            error!("Failed to open image {full_path}, reason {e}");
                         }
                     };
-                    break 'czystka;
                 }
-                break 'czystka;
-            }
+                Err(e) => {
+                    error!("Failed to open image {full_path}, reason {e}");
+                }
+            };
         } else {
             match Pixbuf::from_file(&full_path) {
                 Ok(t) => {
                     pixbuf = t;
                 }
                 Err(e) => {
-                    println!("Failed to open image {full_path}, reason {e}");
+                    error!("Failed to open image {full_path}, reason {e}");
                 }
             };
         }
@@ -394,11 +427,11 @@ fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>)
         #[allow(clippy::never_loop)]
         loop {
             let Some(pixbuf_big) = resize_pixbuf_dimension(&pixbuf, (BIG_PREVIEW_SIZE, BIG_PREVIEW_SIZE), InterpType::Bilinear) else {
-                println!("Failed to resize image {full_path}.");
+                error!("Failed to resize image {full_path}.");
                 break;
             };
             let Some(pixbuf_small) = resize_pixbuf_dimension(&pixbuf_big, (SMALL_PREVIEW_SIZE, SMALL_PREVIEW_SIZE), InterpType::Bilinear) else {
-                println!("Failed to resize image {full_path}.");
+                error!("Failed to resize image {full_path}.");
                 break;
             };
 
@@ -413,7 +446,7 @@ fn generate_cache_for_results(vector_with_path: Vec<(String, String, TreePath)>)
 }
 
 fn get_all_path(model: &TreeModel, current_path: &TreePath, column_header: i32, column_path: i32, column_name: i32) -> Vec<(String, String, TreePath)> {
-    let used_iter = model.iter(current_path).unwrap();
+    let used_iter = model.iter(current_path).expect("Using invalid tree_path");
 
     assert!(model.get::<bool>(&used_iter, column_header));
     let using_reference = !model.get::<String>(&used_iter, column_path).is_empty();
@@ -454,7 +487,7 @@ fn get_all_path(model: &TreeModel, current_path: &TreePath, column_header: i32,
 }
 
 fn move_iter(model: &TreeModel, tree_path: &TreePath, column_header: i32, go_next: bool) -> TreePath {
-    let tree_iter = model.iter(tree_path).unwrap();
+    let tree_iter = model.iter(tree_path).expect("Using invalid tree_path");
 
     assert!(model.get::<bool>(&tree_iter, column_header));
 
@@ -507,7 +540,7 @@ fn populate_similar_scrolled_view(
         let smaller_box = gtk4::Box::new(Orientation::Horizontal, 2);
 
         let button_left = gtk4::Button::builder().label(&flg!("compare_move_left_button")).build();
-        let label = gtk4::Label::builder().label(&(number + 1).to_string()).build();
+        let label = gtk4::Label::builder().label((number + 1).to_string()).build();
         let button_right = gtk4::Button::builder().label(&flg!("compare_move_right_button")).build();
 
         let image_compare_left = image_compare_left.clone();
@@ -527,7 +560,7 @@ fn populate_similar_scrolled_view(
             update_bottom_buttons(&all_gtk_box_clone, &shared_using_for_preview_clone, &shared_image_cache_clone);
             image_compare_left.set_paintable(big_thumbnail_clone.paintable().as_ref());
 
-            let is_active = model_clone.get::<bool>(&model_clone.iter(&tree_path_clone).unwrap(), column_selection);
+            let is_active = model_clone.get::<bool>(&model_clone.iter(&tree_path_clone).expect("Invalid tree_path"), column_selection);
             check_button_left_preview_text_clone.set_active(is_active);
             check_button_left_preview_text_clone.set_label(Some(&format!("{}. {}", number + 1, get_max_file_name(&path_clone, 60))));
         });
@@ -546,7 +579,7 @@ fn populate_similar_scrolled_view(
             update_bottom_buttons(&all_gtk_box_clone, &shared_using_for_preview_clone, &shared_image_cache_clone);
             image_compare_right.set_paintable(big_thumbnail_clone.paintable().as_ref());
 
-            let is_active = model_clone.get::<bool>(&model_clone.iter(&tree_path_clone).unwrap(), column_selection);
+            let is_active = model_clone.get::<bool>(&model_clone.iter(&tree_path_clone).expect("Invalid tree_path"), column_selection);
             check_button_right_preview_text_clone.set_active(is_active);
             check_button_right_preview_text_clone.set_label(Some(&format!("{}. {}", number + 1, get_max_file_name(&path_clone, 60))));
         });
@@ -581,15 +614,15 @@ fn update_bottom_buttons(
     shared_using_for_preview: &Rc<RefCell<(Option<TreePath>, Option<TreePath>)>>,
     image_cache: &Rc<RefCell<Vec<(String, String, Image, Image, TreePath)>>>,
 ) {
-    let left_tree_view = (shared_using_for_preview.borrow()).0.clone().unwrap();
-    let right_tree_view = (shared_using_for_preview.borrow()).1.clone().unwrap();
+    let left_tree_view = shared_using_for_preview.borrow().0.clone().expect("Left tree_view not set");
+    let right_tree_view = shared_using_for_preview.borrow().1.clone().expect("Right tree_view not set");
 
     for (number, i) in get_all_direct_children(all_gtk_box).into_iter().enumerate() {
         let cache_tree_path = (*image_cache.borrow())[number].4.clone();
         let is_chosen = cache_tree_path != right_tree_view && cache_tree_path != left_tree_view;
 
-        let bx = i.downcast::<gtk4::Box>().unwrap();
-        let smaller_bx = bx.first_child().unwrap().downcast::<gtk4::Box>().unwrap();
+        let bx = i.downcast::<gtk4::Box>().expect("Not Box");
+        let smaller_bx = bx.first_child().expect("No first child").downcast::<gtk4::Box>().expect("First child is not Box");
         for items in get_all_direct_children(&smaller_bx) {
             if let Ok(btn) = items.downcast::<gtk4::Button>() {
                 btn.set_sensitive(is_chosen);
@@ -606,7 +639,7 @@ fn get_current_group_and_iter_from_selection(model: &TreeModel, selection: &Tree
 
     let selected_records = selection.selected_rows().0;
 
-    let iter = model.iter_first().unwrap(); // Checking that treeview is not empty should be done before
+    let iter = model.iter_first().expect("Model is no empty, so should have first item"); // Checking that treeview is not empty should be done before
     header_clone = iter; // if nothing selected, use first group
     possible_header = iter; // if nothing selected, use first group
     assert!(model.get::<bool>(&iter, column_header)); // First element should be header
