@@ -1,9 +1,10 @@
 use std::collections::BTreeMap;
 use std::fs;
-use std::fs::FileType;
 
+use czkawka_core::common::check_if_folder_contains_only_empty_folders;
 use gtk4::prelude::*;
 use gtk4::{Align, CheckButton, Dialog, Orientation, ResponseType, TextView};
+use log::debug;
 
 use crate::flg;
 use crate::gui_structs::gui_data::GuiData;
@@ -13,7 +14,7 @@ use crate::notebook_info::NOTEBOOKS_INFO;
 
 // TODO add support for checking if really symlink doesn't point to correct directory/file
 
-pub fn connect_button_delete(gui_data: &GuiData) {
+pub(crate) fn connect_button_delete(gui_data: &GuiData) {
     let buttons_delete = gui_data.bottom_buttons.buttons_delete.clone();
 
     let gui_data = gui_data.clone(); // TODO this maybe can be replaced, not sure if worth to clone everything
@@ -261,7 +262,7 @@ pub async fn check_if_deleting_all_files_in_group(
     false
 }
 
-pub fn empty_folder_remover(
+pub(crate) fn empty_folder_remover(
     tree_view: &gtk4::TreeView,
     column_file_name: i32,
     column_path: i32,
@@ -290,6 +291,10 @@ pub fn empty_folder_remover(
         return; // No selected rows
     }
 
+    debug!("Starting to delete {} folders", selected_rows.len());
+    let start_time = std::time::Instant::now();
+    let mut deleted_folders: u32 = 0;
+
     let mut messages: String = String::new();
 
     // Must be deleted from end to start, because when deleting entries, TreePath(and also TreeIter) will points to invalid data
@@ -298,80 +303,44 @@ pub fn empty_folder_remover(
 
         let name = model.get::<String>(&iter, column_file_name);
         let path = model.get::<String>(&iter, column_path);
+        let full_path = get_full_name_from_path_name(&path, &name);
 
         // We must check if folder is really empty or contains only other empty folders
-        let mut error_happened = false;
-        let mut folders_to_check: Vec<String> = vec![get_full_name_from_path_name(&path, &name)];
-        let mut next_folder: String;
-        'dir: while let Some(current_folder) = folders_to_check.pop() {
-            let read_dir = match fs::read_dir(&current_folder) {
-                Ok(t) => t,
-                Err(_inspected) => {
-                    error_happened = true;
-                    break 'dir;
-                }
-            };
-
-            for entry in read_dir {
-                let entry_data = match entry {
-                    Ok(t) => t,
-                    Err(_inspected) => {
-                        error_happened = true;
-                        break 'dir;
-                    }
-                };
-                let file_type: FileType = match entry_data.file_type() {
-                    Ok(t) => t,
-                    Err(_inspected) => {
-                        error_happened = true;
-                        break 'dir;
-                    }
-                };
-                if file_type.is_dir() {
-                    next_folder = String::new()
-                        + &current_folder
-                        + "/"
-                        + match &entry_data.file_name().into_string() {
-                            Ok(t) => t,
-                            Err(_inspected) => {
-                                error_happened = true;
-                                break 'dir;
-                            }
-                        };
-                    folders_to_check.push(next_folder.clone());
-                } else {
-                    error_happened = true;
-                }
-            }
-        }
+        let mut error_happened = check_if_folder_contains_only_empty_folders(&full_path).is_err();
 
         if !error_happened {
             if !use_trash {
-                match fs::remove_dir_all(get_full_name_from_path_name(&path, &name)) {
+                match fs::remove_dir_all(&full_path) {
                     Ok(()) => {
                         model.remove(&iter);
+                        deleted_folders += 1;
                     }
                     Err(_inspected) => error_happened = true,
                 }
             } else {
-                match trash::delete(get_full_name_from_path_name(&path, &name)) {
+                match trash::delete(&full_path) {
                     Ok(()) => {
                         model.remove(&iter);
+                        deleted_folders += 1;
                     }
                     Err(_inspected) => error_happened = true,
                 }
             }
         }
+
+        // This could be changed to add more specific error message, what exactly happened
         if error_happened {
-            messages += &flg!("delete_folder_failed", dir = get_full_name_from_path_name(&path, &name));
+            messages += &flg!("delete_folder_failed", dir = full_path);
             messages += "\n";
         }
     }
 
+    debug!("Deleted {deleted_folders} folders in {:?}", start_time.elapsed());
+
     text_view_errors.buffer().set_text(messages.as_str());
 }
 
-pub fn basic_remove(
+pub(crate) fn basic_remove(
     tree_view: &gtk4::TreeView,
     column_file_name: i32,
     column_path: i32,
@@ -403,6 +372,10 @@ pub fn basic_remove(
         return; // No selected rows
     }
 
+    debug!("Starting to delete {} files", selected_rows.len());
+    let start_time = std::time::Instant::now();
+    let mut deleted_files: u32 = 0;
+
     // Must be deleted from end to start, because when deleting entries, TreePath(and also TreeIter) will points to invalid data
     for tree_path in selected_rows.iter().rev() {
         let iter = model.iter(tree_path).expect("Using invalid tree_path");
@@ -414,6 +387,7 @@ pub fn basic_remove(
             match fs::remove_file(get_full_name_from_path_name(&path, &name)) {
                 Ok(()) => {
                     model.remove(&iter);
+                    deleted_files += 1;
                 }
 
                 Err(e) => {
@@ -425,6 +399,7 @@ pub fn basic_remove(
             match trash::delete(get_full_name_from_path_name(&path, &name)) {
                 Ok(()) => {
                     model.remove(&iter);
+                    deleted_files += 1;
                 }
                 Err(e) => {
                     messages += flg!("delete_file_failed", name = get_full_name_from_path_name(&path, &name), reason = e.to_string()).as_str();
@@ -434,11 +409,13 @@ pub fn basic_remove(
         }
     }
 
+    debug!("Deleted {deleted_files} files in {:?}", start_time.elapsed());
+
     text_view_errors.buffer().set_text(messages.as_str());
 }
 
 // Remove all occurrences - remove every element which have same path and name as even non selected ones
-pub fn tree_remove(
+pub(crate) fn tree_remove(
     tree_view: &gtk4::TreeView,
     column_file_name: i32,
     column_path: i32,
