@@ -1,13 +1,14 @@
 use gtk4::prelude::*;
 use i18n_embed::DesktopLanguageRequester;
 use i18n_embed::unic_langid::LanguageIdentifier;
+use log::error;
 
 use crate::language_functions::get_language_from_combo_box_text;
 use crate::{GuiData, LANGUAGES_ALL, localizer_gui};
 
 // use i18n_embed::{DesktopLanguageRequester, Localizer};
 
-pub fn connect_change_language(gui_data: &GuiData) {
+pub(crate) fn connect_change_language(gui_data: &GuiData) {
     change_language(gui_data);
 
     let combo_box_settings_language = gui_data.settings.combo_box_settings_language.clone();
@@ -28,19 +29,19 @@ fn change_language(gui_data: &GuiData) {
     let lang_identifier = vec![LanguageIdentifier::from_bytes(lang_short.as_bytes()).expect("Failed to create LanguageIdentifier")];
     for (lib, localizer) in localizers {
         if let Err(error) = localizer.select(&lang_identifier) {
-            eprintln!("Error while loadings languages for {lib} {error:?}");
+            error!("Error while loading languages for {lib} {error:?}");
         }
     }
     gui_data.update_language();
 }
 
-pub fn load_system_language(gui_data: &GuiData) {
+pub(crate) fn load_system_language(gui_data: &GuiData) {
     let requested_languages = DesktopLanguageRequester::requested_languages();
 
     if let Some(language) = requested_languages.first() {
         let old_short_lang = language.to_string();
         let mut short_lang = String::new();
-        // removes from e.g. en_zb, ending _zd since Czkawka don't support this(maybe could add this in future, but only when)
+        // removes from e.g. en_zb, ending _zd since Czkawka doesn't support this (maybe could add this in future)
         for i in old_short_lang.chars() {
             if i.is_ascii_alphabetic() {
                 short_lang.push(i);
@@ -48,18 +49,11 @@ pub fn load_system_language(gui_data: &GuiData) {
                 break;
             }
         }
-        // let mut found: bool = false;
         for (index, lang) in LANGUAGES_ALL.iter().enumerate() {
             if lang.short_text == short_lang {
-                // found = true;
                 gui_data.settings.combo_box_settings_language.set_active(Some(index as u32));
                 break;
             }
         }
-        // if found {
-        //     println!("INFO: Default system language {} is available, so choosing them", short_lang);
-        // } else {
-        //     println!("INFO: Default system language {} is not available, using English(en) instead", short_lang);
-        // }
     }
 }
