@@ -1,7 +1,8 @@
+use czkawka_core::common::items::new_excluded_item;
 use czkawka_core::common::regex_check;
-use czkawka_core::common_items::new_excluded_item;
 use gtk4::prelude::*;
 use gtk4::{ResponseType, TreeIter, Window};
+use log::error;
 use regex::Regex;
 
 use crate::flg;
@@ -388,7 +389,7 @@ fn popover_custom_select_unselect(
                         if let Ok(t) = Regex::new(&regex_wildcard) {
                             t
                         } else {
-                            eprintln!("What? Regex should compile properly.");
+                            error!("What? Regex should compile properly.");
                             confirmation_dialog_select_unselect.close();
                             return;
                         }
@@ -404,15 +405,17 @@ fn popover_custom_select_unselect(
                         confirmation_dialog_select_unselect.close();
                         return;
                     };
+                    let using_reference_folders = column_header.is_some_and(|e| model.get::<bool>(&iter, e)) && !model.get::<String>(&iter, column_file_name).is_empty();
 
                     let mut number_of_all_things = 0;
                     let mut number_of_already_selected_things = 0;
                     let mut vec_of_iters: Vec<TreeIter> = Vec::new();
                     loop {
+                        // If went to header and all previous items were selected, then deselect last item
                         if let Some(column_header) = column_header {
                             if model.get::<bool>(&iter, column_header) {
                                 if select_things {
-                                    if check_all_selected && (number_of_all_things - number_of_already_selected_things == vec_of_iters.len()) {
+                                    if !using_reference_folders && check_all_selected && (number_of_all_things - number_of_already_selected_things == vec_of_iters.len()) {
                                         vec_of_iters.pop();
                                     }
                                     for iter in vec_of_iters {
@@ -485,9 +488,10 @@ fn popover_custom_select_unselect(
                             }
                         }
 
+                        // If went to last item and all previous items were selected, then deselect last item
                         if !model.iter_next(&iter) {
                             if select_things {
-                                if check_all_selected && (number_of_all_things - number_of_already_selected_things == vec_of_iters.len()) {
+                                if !using_reference_folders && check_all_selected && (number_of_all_things - number_of_already_selected_things == vec_of_iters.len()) {
                                     vec_of_iters.pop();
                                 }
                                 for iter in vec_of_iters {
@@ -599,7 +603,7 @@ fn popover_all_except_biggest_smallest(
     popover.popdown();
 }
 
-pub fn connect_popover_select(gui_data: &GuiData) {
+pub(crate) fn connect_popover_select(gui_data: &GuiData) {
     let popover_select = gui_data.popovers_select.popover_select.clone();
     let buttons_popover_select_all = gui_data.popovers_select.buttons_popover_select_all.clone();
     let notebook_main = gui_data.main_notebook.notebook_main.clone();
