@@ -118,7 +118,8 @@ impl GuiData {
         window_main.set_title(Some(&flg!("window_main_title")));
         window_main.show();
 
-        let pixbuf = Pixbuf::from_read(BufReader::new(ICON_ABOUT)).expect("Couldn't load icon");
+        let pixbuf = Pixbuf::from_read(BufReader::new(ICON_ABOUT))
+            .unwrap_or(Pixbuf::new(gdk4::gdk_pixbuf::Colorspace::Rgb, false, 8, 1, 1).expect("Crash is a lot of less likely than loading png file"));
 
         window_main.set_application(Some(application));
 
@@ -195,7 +196,7 @@ impl GuiData {
         scrolled_window_errors.show(); // Not sure why needed, but without it text view errors sometimes hide itself
 
         // Used for sending stop signal to thread
-        let stop_flag = Arc::new(AtomicBool::new(false));
+        let stop_flag = Arc::default();
 
         Self {
             window_main,
@@ -232,7 +233,7 @@ impl GuiData {
         }
     }
 
-    pub fn update_language(&self) {
+    pub(crate) fn update_language(&self) {
         self.window_main.set_title(Some(&flg!("window_main_title")));
 
         self.main_notebook.update_language();
