@@ -1,7 +1,7 @@
 use gtk4::prelude::*;
 
 use crate::help_functions::set_icon_of_button;
-use crate::{flg, CZK_ICON_INFO, CZK_ICON_SETTINGS};
+use crate::{CZK_ICON_INFO, CZK_ICON_SETTINGS, flg};
 
 #[derive(Clone)]
 pub struct GuiHeader {
@@ -10,9 +10,9 @@ pub struct GuiHeader {
 }
 
 impl GuiHeader {
-    pub fn create_from_builder(builder: &gtk4::Builder) -> Self {
-        let button_settings: gtk4::Button = builder.object("button_settings").unwrap();
-        let button_app_info: gtk4::Button = builder.object("button_app_info").unwrap();
+    pub(crate) fn create_from_builder(builder: &gtk4::Builder) -> Self {
+        let button_settings: gtk4::Button = builder.object("button_settings").expect("Cambalache");
+        let button_app_info: gtk4::Button = builder.object("button_app_info").expect("Cambalache");
 
         set_icon_of_button(&button_settings, CZK_ICON_SETTINGS);
         set_icon_of_button(&button_app_info, CZK_ICON_INFO);
@@ -20,7 +20,7 @@ impl GuiHeader {
         Self { button_settings, button_app_info }
     }
 
-    pub fn update_language(&self) {
+    pub(crate) fn update_language(&self) {
         self.button_settings.set_tooltip_text(Some(&flg!("header_setting_button_tooltip")));
         self.button_app_info.set_tooltip_text(Some(&flg!("header_about_button_tooltip")));
     }
