@@ -1,18 +1,18 @@
 use std::collections::HashMap;
+use std::env;
 use std::ffi::OsString;
 use std::fs::File;
 use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
-use std::{env, fs};
 
-use directories_next::ProjectDirs;
+use czkawka_core::common::config_cache_path::get_config_cache_path;
+use czkawka_core::common::get_all_available_threads;
+use czkawka_core::common::items::DEFAULT_EXCLUDED_ITEMS;
+use czkawka_core::common::model::CheckingMethod;
+use czkawka_core::tools::similar_images::SIMILAR_VALUES;
 use gtk4::prelude::*;
 use gtk4::{ComboBoxText, ScrolledWindow, TextView, TreeView};
-
-use czkawka_core::common::get_all_available_threads;
-use czkawka_core::common_dir_traversal::CheckingMethod;
-use czkawka_core::common_items::DEFAULT_EXCLUDED_ITEMS;
-use czkawka_core::similar_images::SIMILAR_VALUES;
+use log::error;
 
 use crate::flg;
 use crate::gui_structs::gui_main_notebook::GuiMainNotebook;
@@ -20,7 +20,7 @@ use crate::gui_structs::gui_settings::GuiSettings;
 use crate::gui_structs::gui_upper_notebook::GuiUpperNotebook;
 use crate::help_combo_box::DUPLICATES_CHECK_METHOD_COMBO_BOX;
 use crate::help_functions::*;
-use crate::language_functions::{get_language_from_combo_box_text, LANGUAGES_ALL};
+use crate::language_functions::{LANGUAGES_ALL, get_language_from_combo_box_text};
 
 const SAVE_FILE_NAME: &str = "czkawka_gui_config_4.txt";
 
@@ -43,6 +43,10 @@ const DEFAULT_IMAGE_REMOVE_AUTO_OUTDATED_CACHE: bool = true;
 const DEFAULT_DUPLICATE_REMOVE_AUTO_OUTDATED_CACHE: bool = true;
 const DEFAULT_DUPLICATE_CASE_SENSITIVE_NAME_CHECKING: bool = false;
 const DEFAULT_GENERAL_IGNORE_OTHER_FILESYSTEMS: bool = false;
+const DEFAULT_USING_RUST_LIBRARIES_TO_SHOW_PREVIEW: bool = true;
+
+const DEFAULT_MUSIC_APPROXIMATE_COMPARISON: bool = false;
+const DEFAULT_MUSIC_GROUP_CONTENT_BY_TITLE: bool = false;
 
 const DEFAULT_BROKEN_FILES_PDF: bool = true;
 const DEFAULT_BROKEN_FILES_AUDIO: bool = true;
@@ -61,7 +65,7 @@ pub const DEFAULT_MINIMAL_FILE_SIZE: &str = "16384";
 pub const DEFAULT_MAXIMAL_FILE_SIZE: &str = "999999999999";
 
 #[cfg(target_family = "unix")]
-const DEFAULT_EXCLUDED_DIRECTORIES: &[&str] = &["/proc", "/dev", "/sys", "/run", "/snap"];
+const DEFAULT_EXCLUDED_DIRECTORIES: &[&str] = &["/proc", "/dev", "/sys", "/snap"];
 #[cfg(not(target_family = "unix"))]
 const DEFAULT_EXCLUDED_DIRECTORIES: &[&str] = &["C:\\Windows"];
 
@@ -71,14 +75,14 @@ struct LoadSaveStruct {
 }
 
 impl LoadSaveStruct {
-    pub fn with_text_view(text_view: TextView) -> Self {
+    pub(crate) fn with_text_view(text_view: TextView) -> Self {
         Self {
             loaded_items: Default::default(),
             text_view,
         }
     }
 
-    pub fn get_vector_string(&self, key: &str, default_value: Vec<String>) -> Vec<String> {
+    pub(crate) fn get_vector_string(&self, key: &str, default_value: Vec<String>) -> Vec<String> {
         if self.loaded_items.contains_key(key) {
             let mut new_vector = Vec::new();
             for i in &self.loaded_items[key] {
@@ -91,18 +95,16 @@ impl LoadSaveStruct {
 
         default_value
     }
-    pub fn get_integer_string(&self, key: String, default_value: String) -> String {
-        if default_value.parse::<i64>().is_err() {
-            println!("Default value {default_value} can't be convert to integer value");
-            panic!();
-        }
+    pub(crate) fn get_integer_string(&self, key: String, default_value: String) -> String {
+        // This is safe, because default value is const value
+        assert!(default_value.parse::<i64>().is_ok(), "Default value {default_value} can't be convert to integer value");
         let mut returned_value = self.get_string(key, default_value.clone());
         if returned_value.parse::<i64>().is_err() {
             returned_value = default_value;
         }
         returned_value
     }
-    pub fn get_string(&self, key: String, default_value: String) -> String {
+    pub(crate) fn get_string(&self, key: String, default_value: String) -> String {
         if self.loaded_items.contains_key(&key) {
             let item = &self.loaded_items[&key].clone().into_iter().filter(|e| !e.is_empty()).collect::<Vec<String>>();
             return if item.len() == 1 {
@@ -117,7 +119,7 @@ impl LoadSaveStruct {
 
         default_value
     }
-    pub fn get_object<T: std::str::FromStr>(&self, key: String, default_value: T) -> T {
+    pub(crate) fn get_object<T: std::str::FromStr>(&self, key: String, default_value: T) -> T {
         if self.loaded_items.contains_key(&key) {
             let item = &self.loaded_items[&key].clone().into_iter().filter(|e| !e.is_empty()).collect::<Vec<String>>();
 
@@ -125,7 +127,7 @@ impl LoadSaveStruct {
                 if let Ok(t) = item[0].parse::<T>() {
                     t
                 } else {
-                    println!("Failed to decode integer from \"{}\", found {:?}", key, item[0]);
+                    error!("Failed to decode integer from \"{}\", found {:?}", key, item[0]);
                     default_value
                 }
             } else {
@@ -136,7 +138,7 @@ impl LoadSaveStruct {
 
         default_value
     }
-    pub fn get_bool(&self, key: String, default_value: bool) -> bool {
+    pub(crate) fn get_bool(&self, key: String, default_value: bool) -> bool {
         if self.loaded_items.contains_key(&key) {
             let item = &self.loaded_items[&key].clone().into_iter().filter(|e| !e.is_empty()).collect::<Vec<String>>();
             return if item.len() == 1 {
@@ -159,7 +161,7 @@ impl LoadSaveStruct {
     }
 
     // Bool, int, string
-    pub fn save_var<T: ToString>(&mut self, key: String, value: &T) {
+    pub(crate) fn save_var<T: ToString>(&mut self, key: String, value: &T) {
         if self.loaded_items.contains_key(&key) {
             add_text_to_text_view(&self.text_view, &flg!("saving_loading_saving_same_keys", key = key.clone()));
         }
@@ -167,7 +169,7 @@ impl LoadSaveStruct {
         self.loaded_items.insert(key, vec![value.to_string()]);
     }
 
-    pub fn save_list_store(&mut self, key: String, tree_view: &TreeView, column_path: i32) {
+    pub(crate) fn save_list_store(&mut self, key: String, tree_view: &TreeView, column_path: i32) {
         let mut vec_string = vec![];
         let list_store = get_list_store(tree_view);
         if let Some(iter) = list_store.iter_first() {
@@ -182,64 +184,12 @@ impl LoadSaveStruct {
         self.loaded_items.insert(key, vec_string);
     }
 
-    pub fn open_save_file(&self, text_view_errors: &TextView, save_configuration: bool, manual_execution: bool) -> Option<(File, PathBuf)> {
-        if let Some(proj_dirs) = ProjectDirs::from("pl", "Qarmin", "Czkawka") {
-            // Lin: /home/username/.config/czkawka
-            // Win: C:\Users\Username\AppData\Roaming\Qarmin\Czkawka\config
-            // Mac: /Users/Username/Library/Application Support/pl.Qarmin.Czkawka
+    pub(crate) fn open_save_file(&self, text_view_errors: &TextView, save_configuration: bool, manual_execution: bool) -> Option<(File, PathBuf)> {
+        let config_dir = get_config_cache_path()?.config_folder;
+        let config_file = config_dir.join(Path::new(SAVE_FILE_NAME));
 
-            let config_dir = proj_dirs.config_dir();
-            let config_file = config_dir.join(Path::new(SAVE_FILE_NAME));
-
-            if save_configuration {
-                if config_dir.exists() {
-                    if !config_dir.is_dir() {
-                        add_text_to_text_view(
-                            text_view_errors,
-                            &flg!("saving_loading_folder_config_instead_file", path = config_dir.to_string_lossy().to_string()),
-                        );
-                        return None;
-                    }
-                } else if let Err(e) = fs::create_dir_all(config_dir) {
-                    add_text_to_text_view(
-                        text_view_errors,
-                        &flg!(
-                            "saving_loading_failed_to_create_configuration_folder",
-                            path = config_dir.to_string_lossy().to_string(),
-                            reason = e.to_string()
-                        ),
-                    );
-                    return None;
-                }
-
-                let config_file_handler = match File::create(&config_file) {
-                    Ok(t) => t,
-                    Err(e) => {
-                        add_text_to_text_view(
-                            text_view_errors,
-                            &flg!(
-                                "saving_loading_failed_to_create_config_file",
-                                path = config_file.to_string_lossy().to_string(),
-                                reason = e.to_string()
-                            ),
-                        );
-                        return None;
-                    }
-                };
-                return Some((config_file_handler, config_file));
-            }
-            if !config_file.is_file() {
-                if manual_execution {
-                    // Don't show errors when there is no configuration file when starting app
-                    add_text_to_text_view(
-                        text_view_errors,
-                        &flg!("saving_loading_failed_to_read_config_file", path = config_file.to_string_lossy().to_string()),
-                    );
-                }
-                return None;
-            }
-
-            let config_file_handler = match File::open(&config_file) {
+        if save_configuration {
+            let config_file_handler = match File::create(&config_file) {
                 Ok(t) => t,
                 Err(e) => {
                     add_text_to_text_view(
@@ -255,12 +205,35 @@ impl LoadSaveStruct {
             };
             return Some((config_file_handler, config_file));
         }
-        add_text_to_text_view(text_view_errors, flg!("saving_loading_failed_to_get_home_directory").as_str());
+        if !config_file.is_file() {
+            if manual_execution {
+                // Don't show errors when there is no configuration file when starting app
+                add_text_to_text_view(
+                    text_view_errors,
+                    &flg!("saving_loading_failed_to_read_config_file", path = config_file.to_string_lossy().to_string()),
+                );
+            }
+            return None;
+        }
 
-        None
+        let config_file_handler = match File::open(&config_file) {
+            Ok(t) => t,
+            Err(e) => {
+                add_text_to_text_view(
+                    text_view_errors,
+                    &flg!(
+                        "saving_loading_failed_to_create_config_file",
+                        path = config_file.to_string_lossy().to_string(),
+                        reason = e.to_string()
+                    ),
+                );
+                return None;
+            }
+        };
+        Some((config_file_handler, config_file))
     }
 
-    pub fn open_and_read_content(&mut self, text_view_errors: &TextView, manual_execution: bool) {
+    pub(crate) fn open_and_read_content(&mut self, text_view_errors: &TextView, manual_execution: bool) {
         if let Some((mut config_file_handler, config_file)) = self.open_save_file(text_view_errors, false, manual_execution) {
             let mut loaded_data: String = String::new();
             if let Err(e) = config_file_handler.read_to_string(&mut loaded_data) {
@@ -301,7 +274,7 @@ impl LoadSaveStruct {
         }
     }
 
-    pub fn save_to_file(&self, text_view_errors: &TextView) {
+    pub(crate) fn save_to_file(&self, text_view_errors: &TextView) {
         if let Some((mut config_file_handler, config_file)) = self.open_save_file(text_view_errors, true, false) {
             let mut data_saved: bool = false;
             for (key, vec_string) in &self.loaded_items {
@@ -380,12 +353,14 @@ enum LoadText {
     SimilarVideosSimilarity,
     SimilarVideosIgnoreSameSize,
     MusicApproximateComparison,
+    MusicCompareByTitle,
     DuplicateNameCaseSensitive,
     BrokenFilesPdf,
     BrokenFilesAudio,
     BrokenFilesImage,
     BrokenFilesArchive,
     ThreadNumber,
+    GeneralUseRustLibrariesToPreview,
 }
 
 fn create_hash_map() -> (HashMap<LoadText, String>, HashMap<String, LoadText>) {
@@ -433,6 +408,8 @@ fn create_hash_map() -> (HashMap<LoadText, String>, HashMap<String, LoadText>) {
         (LoadText::BrokenFilesArchive, "broken_files_archive"),
         (LoadText::GeneralIgnoreOtherFilesystems, "ignore_other_filesystems"),
         (LoadText::ThreadNumber, "thread_number"),
+        (LoadText::MusicCompareByTitle, "music_compare_by_title"),
+        (LoadText::GeneralUseRustLibrariesToPreview, "use_rust_libraries_to_preview"),
     ];
     let mut hashmap_ls: HashMap<LoadText, String> = Default::default();
     let mut hashmap_sl: HashMap<String, LoadText> = Default::default();
@@ -445,7 +422,7 @@ fn create_hash_map() -> (HashMap<LoadText, String>, HashMap<String, LoadText>) {
     (hashmap_ls, hashmap_sl)
 }
 
-pub fn save_configuration(manual_execution: bool, upper_notebook: &GuiUpperNotebook, main_notebook: &GuiMainNotebook, settings: &GuiSettings, text_view_errors: &TextView) {
+pub(crate) fn save_configuration(manual_execution: bool, upper_notebook: &GuiUpperNotebook, main_notebook: &GuiMainNotebook, settings: &GuiSettings, text_view_errors: &TextView) {
     let check_button_settings_save_at_exit = settings.check_button_settings_save_at_exit.clone();
     let text_view_errors = text_view_errors.clone();
 
@@ -523,6 +500,10 @@ pub fn save_configuration(manual_execution: bool, upper_notebook: &GuiUpperNoteb
         hashmap_ls[&LoadText::GeneralIgnoreOtherFilesystems].clone(),
         &settings.check_button_settings_one_filesystem.is_active(),
     );
+    saving_struct.save_var(
+        hashmap_ls[&LoadText::GeneralUseRustLibrariesToPreview].clone(),
+        &settings.check_button_settings_use_rust_preview.is_active(),
+    );
 
     saving_struct.save_var(
         hashmap_ls[&LoadText::BrokenFilesArchive].clone(),
@@ -541,33 +522,33 @@ pub fn save_configuration(manual_execution: bool, upper_notebook: &GuiUpperNoteb
     );
     saving_struct.save_var(
         hashmap_ls[&LoadText::Language].clone(),
-        &get_language_from_combo_box_text(&settings.combo_box_settings_language.active_text().unwrap()).short_text,
+        &get_language_from_combo_box_text(&settings.combo_box_settings_language.active_text().expect("No active text")).short_text,
     );
 
     // Comboboxes main notebook
     saving_struct.save_var(
         hashmap_ls[&LoadText::ComboBoxDuplicateHashType].clone(),
-        &main_notebook.combo_box_duplicate_hash_type.active().unwrap_or(0),
+        &main_notebook.combo_box_duplicate_hash_type.active().unwrap_or_default(),
     );
     saving_struct.save_var(
         hashmap_ls[&LoadText::ComboBoxDuplicateCheckMethod].clone(),
-        &main_notebook.combo_box_duplicate_check_method.active().unwrap_or(0),
+        &main_notebook.combo_box_duplicate_check_method.active().unwrap_or_default(),
     );
     saving_struct.save_var(
         hashmap_ls[&LoadText::ComboBoxImageResizeAlgorithm].clone(),
-        &main_notebook.combo_box_image_resize_algorithm.active().unwrap_or(0),
+        &main_notebook.combo_box_image_resize_algorithm.active().unwrap_or_default(),
     );
     saving_struct.save_var(
         hashmap_ls[&LoadText::ComboBoxImageHashType].clone(),
-        &main_notebook.combo_box_image_hash_algorithm.active().unwrap_or(0),
+        &main_notebook.combo_box_image_hash_algorithm.active().unwrap_or_default(),
     );
     saving_struct.save_var(
         hashmap_ls[&LoadText::ComboBoxImageHashSize].clone(),
-        &main_notebook.combo_box_image_hash_size.active().unwrap_or(0),
+        &main_notebook.combo_box_image_hash_size.active().unwrap_or_default(),
     );
     saving_struct.save_var(
         hashmap_ls[&LoadText::ComboBoxBigFiles].clone(),
-        &main_notebook.combo_box_big_files_mode.active().unwrap_or(0),
+        &main_notebook.combo_box_big_files_mode.active().unwrap_or_default(),
     );
 
     // Other2
@@ -596,11 +577,15 @@ pub fn save_configuration(manual_execution: bool, upper_notebook: &GuiUpperNoteb
         hashmap_ls[&LoadText::MusicApproximateComparison].clone(),
         &main_notebook.check_button_music_approximate_comparison.is_active(),
     );
+    saving_struct.save_var(
+        hashmap_ls[&LoadText::MusicCompareByTitle].clone(),
+        &main_notebook.check_button_music_compare_only_in_title_group.is_active(),
+    );
 
     saving_struct.save_to_file(&text_view_errors);
 }
 
-pub fn load_configuration(
+pub(crate) fn load_configuration(
     manual_execution: bool,
     upper_notebook: &GuiUpperNotebook,
     main_notebook: &GuiMainNotebook,
@@ -617,7 +602,7 @@ pub fn load_configuration(
     loaded_entries.open_and_read_content(&text_view_errors, manual_execution);
 
     // Load here language, default system language could change value in settings so we don't want to lose this value
-    let short_language = get_language_from_combo_box_text(&settings.combo_box_settings_language.active_text().unwrap())
+    let short_language = get_language_from_combo_box_text(&settings.combo_box_settings_language.active_text().expect("No active text"))
         .short_text
         .to_string();
 
@@ -646,6 +631,11 @@ pub fn load_configuration(
     let use_json_cache: bool = loaded_entries.get_bool(hashmap_ls[&LoadText::UseJsonCacheFile].clone(), DEFAULT_SAVE_ALSO_AS_JSON);
     let use_trash: bool = loaded_entries.get_bool(hashmap_ls[&LoadText::DeleteToTrash].clone(), DEFAULT_USE_TRASH);
     let ignore_other_fs: bool = loaded_entries.get_bool(hashmap_ls[&LoadText::GeneralIgnoreOtherFilesystems].clone(), DEFAULT_GENERAL_IGNORE_OTHER_FILESYSTEMS);
+    let use_rust_libraries_to_preview: bool = loaded_entries.get_bool(
+        hashmap_ls[&LoadText::GeneralUseRustLibrariesToPreview].clone(),
+        DEFAULT_USING_RUST_LIBRARIES_TO_SHOW_PREVIEW,
+    );
+
     let delete_outdated_cache_duplicates: bool = loaded_entries.get_bool(
         hashmap_ls[&LoadText::DuplicateDeleteOutdatedCacheEntries].clone(),
         DEFAULT_DUPLICATE_REMOVE_AUTO_OUTDATED_CACHE,
@@ -674,6 +664,8 @@ pub fn load_configuration(
     let similar_videos_similarity = loaded_entries.get_object(hashmap_ls[&LoadText::SimilarVideosSimilarity].clone(), DEFAULT_SIMILAR_VIDEOS_SIMILARITY);
     let similar_videos_ignore_same_size = loaded_entries.get_bool(hashmap_ls[&LoadText::SimilarVideosIgnoreSameSize].clone(), DEFAULT_SIMILAR_VIDEOS_IGNORE_SAME_SIZE);
     let check_button_case_sensitive_name = loaded_entries.get_object(hashmap_ls[&LoadText::DuplicateNameCaseSensitive].clone(), DEFAULT_DUPLICATE_CASE_SENSITIVE_NAME_CHECKING);
+    let check_button_music_approximate_comparison = loaded_entries.get_object(hashmap_ls[&LoadText::MusicApproximateComparison].clone(), DEFAULT_MUSIC_APPROXIMATE_COMPARISON);
+    let check_button_music_compare_by_title = loaded_entries.get_object(hashmap_ls[&LoadText::MusicCompareByTitle].clone(), DEFAULT_MUSIC_GROUP_CONTENT_BY_TITLE);
 
     let check_button_broken_files_archive = loaded_entries.get_object(hashmap_ls[&LoadText::BrokenFilesArchive].clone(), DEFAULT_BROKEN_FILES_ARCHIVE);
     let check_button_broken_files_pdf = loaded_entries.get_object(hashmap_ls[&LoadText::BrokenFilesPdf].clone(), DEFAULT_BROKEN_FILES_PDF);
@@ -723,7 +715,7 @@ pub fn load_configuration(
                 .collect::<Vec<_>>();
 
             if inc_dir.is_empty() {
-                println!("Arguments {arguments:?} should contains at least one directory to include");
+                error!("Arguments {arguments:?} should contains at least one directory to include");
             } else {
                 included_directories = inc_dir;
                 excluded_directories = exc_dir;
@@ -788,6 +780,7 @@ pub fn load_configuration(
         settings.entry_settings_cache_file_minimal_size.set_text(&cache_minimal_size);
         settings.entry_settings_prehash_cache_file_minimal_size.set_text(&cache_prehash_minimal_size);
         settings.check_button_settings_one_filesystem.set_active(ignore_other_fs);
+        settings.check_button_settings_use_rust_preview.set_active(use_rust_libraries_to_preview);
 
         save_proper_value_to_combo_box(&main_notebook.combo_box_duplicate_hash_type, combo_box_duplicate_hash_type);
         save_proper_value_to_combo_box(&main_notebook.combo_box_duplicate_check_method, combo_box_duplicate_checking_method);
@@ -801,6 +794,10 @@ pub fn load_configuration(
         main_notebook.check_button_image_ignore_same_size.set_active(similar_images_ignore_same_size);
         main_notebook.check_button_video_ignore_same_size.set_active(similar_videos_ignore_same_size);
         main_notebook.scale_similarity_similar_videos.set_value(similar_videos_similarity as f64);
+        main_notebook.check_button_music_compare_only_in_title_group.set_active(check_button_music_compare_by_title);
+        main_notebook
+            .check_button_music_approximate_comparison
+            .set_active(check_button_music_approximate_comparison);
 
         main_notebook.check_button_broken_files_audio.set_active(check_button_broken_files_audio);
         main_notebook.check_button_broken_files_pdf.set_active(check_button_broken_files_pdf);
@@ -808,7 +805,7 @@ pub fn load_configuration(
         main_notebook.check_button_broken_files_archive.set_active(check_button_broken_files_archive);
 
         {
-            let combo_chosen_index = main_notebook.combo_box_duplicate_check_method.active().unwrap();
+            let combo_chosen_index = main_notebook.combo_box_duplicate_check_method.active().expect("Failed to get active item");
 
             if DUPLICATES_CHECK_METHOD_COMBO_BOX[combo_chosen_index as usize].check_method == CheckingMethod::Hash {
                 main_notebook.combo_box_duplicate_hash_type.set_visible(true);
@@ -826,7 +823,7 @@ pub fn load_configuration(
         }
 
         // Set size of similarity scale gtk node, must be set BEFORE setting value of this
-        let index = main_notebook.combo_box_image_hash_size.active().unwrap() as usize;
+        let index = main_notebook.combo_box_image_hash_size.active().expect("Failed to get active item") as usize;
 
         main_notebook.scale_similarity_similar_images.set_range(0_f64, SIMILAR_VALUES[index][5] as f64);
         main_notebook.scale_similarity_similar_images.set_fill_level(SIMILAR_VALUES[index][5] as f64);
@@ -872,7 +869,7 @@ fn save_proper_value_to_combo_box(combo_box: &ComboBoxText, what_to_save: u32) {
     }
 }
 
-pub fn reset_configuration(manual_clearing: bool, upper_notebook: &GuiUpperNotebook, main_notebook: &GuiMainNotebook, settings: &GuiSettings, text_view_errors: &TextView) {
+pub(crate) fn reset_configuration(manual_clearing: bool, upper_notebook: &GuiUpperNotebook, main_notebook: &GuiMainNotebook, settings: &GuiSettings, text_view_errors: &TextView) {
     // TODO Maybe add popup dialog to confirm resetting
     let text_view_errors = text_view_errors.clone();
 
@@ -885,7 +882,7 @@ pub fn reset_configuration(manual_clearing: bool, upper_notebook: &GuiUpperNoteb
         list_store.clear();
 
         let current_dir: String = match env::current_dir() {
-            Ok(t) => t.to_str().unwrap().to_string(),
+            Ok(t) => t.to_string_lossy().to_string(),
             Err(_inspected) => {
                 if cfg!(target_family = "unix") {
                     add_text_to_text_view(&text_view_errors, "Failed to read current directory, setting /home instead");
@@ -950,6 +947,7 @@ pub fn reset_configuration(manual_clearing: bool, upper_notebook: &GuiUpperNoteb
         settings.entry_settings_prehash_cache_file_minimal_size.set_text(DEFAULT_PREHASH_MINIMAL_CACHE_SIZE);
         settings.combo_box_settings_language.set_active(Some(0));
         settings.check_button_settings_one_filesystem.set_active(DEFAULT_GENERAL_IGNORE_OTHER_FILESYSTEMS);
+        settings.check_button_settings_use_rust_preview.set_active(DEFAULT_USING_RUST_LIBRARIES_TO_SHOW_PREVIEW);
 
         main_notebook.combo_box_duplicate_hash_type.set_active(Some(0));
         main_notebook.combo_box_duplicate_check_method.set_active(Some(0));
@@ -966,6 +964,12 @@ pub fn reset_configuration(manual_clearing: bool, upper_notebook: &GuiUpperNoteb
         main_notebook.scale_similarity_similar_images.set_range(0_f64, SIMILAR_VALUES[0][5] as f64); // DEFAULT FOR MAX of 8
         main_notebook.scale_similarity_similar_images.set_fill_level(SIMILAR_VALUES[0][5] as f64);
 
+        main_notebook
+            .check_button_music_compare_only_in_title_group
+            .set_active(DEFAULT_MUSIC_GROUP_CONTENT_BY_TITLE);
+
+        main_notebook.check_button_music_approximate_comparison.set_active(DEFAULT_MUSIC_APPROXIMATE_COMPARISON);
+
         main_notebook.entry_big_files_number.set_text(DEFAULT_NUMBER_OF_BIGGEST_FILES);
         main_notebook.scale_similarity_similar_images.set_value(DEFAULT_SIMILAR_IMAGES_SIMILARITY as f64);
         main_notebook.check_button_image_ignore_same_size.set_active(DEFAULT_SIMILAR_IMAGES_IGNORE_SAME_SIZE);
