@@ -3,7 +3,7 @@ use crate::help_functions::get_notebook_enum_from_tree_view;
 use crate::notebook_enums::to_notebook_main_enum;
 use crate::notebook_info::NOTEBOOKS_INFO;
 
-pub fn validate_notebook_data(gui_data: &GuiData) {
+pub(crate) fn validate_notebook_data(gui_data: &GuiData) {
     // Test treeviews names, each treeview should have set name same as variable name
 
     for item in &gui_data.main_notebook.get_main_tree_views() {
