@@ -1,20 +1,10 @@
-use czkawka_core::common::get_config_cache_path;
+use czkawka_core::common::config_cache_path::get_config_cache_path;
 use log::error;
 use slint::ComponentHandle;
 
 use crate::{Callabler, MainWindow};
 
-pub fn connect_open_items(app: &MainWindow) {
-    app.global::<Callabler>().on_item_opened(move |path| {
-        match open::that(&*path) {
-            Ok(()) => {}
-            Err(e) => {
-                eprintln!("Failed to open file: {e}");
-            }
-        };
-        // TODO - this should be added to line edit
-    });
-
+pub(crate) fn connect_open_items(app: &MainWindow) {
     app.global::<Callabler>().on_open_config_folder(move || {
         let Some(config_cache) = get_config_cache_path() else {
             error!("Failed to open config folder");
@@ -39,7 +29,7 @@ pub fn connect_open_items(app: &MainWindow) {
         match open::that(link.as_str()) {
             Ok(()) => {}
             Err(e) => {
-                eprintln!("Failed to open link: {e}");
+                error!("Failed to open link: {e}");
             }
         };
     });
