@@ -1,20 +1,22 @@
 use slint::{ComponentHandle, Model, ModelRc, VecModel};
 
-use crate::common::{
-    connect_i32_into_u64, get_int_height_idx, get_int_modification_date_idx, get_int_size_idx, get_int_width_idx, get_is_header_mode, get_tool_model, set_tool_model,
-};
-use crate::{Callabler, CurrentTab, GuiState, MainListModel, MainWindow, SelectMode, SelectModel};
+use crate::common::connect_i32_into_u64;
+use crate::connect_row_selection::checker::change_number_of_enabled_items;
+use crate::connect_translation::translate_select_mode;
+use crate::{ActiveTab, Callabler, GuiState, MainListModel, MainWindow, SelectMode, SelectModel};
+
+type SelectionResult = (u64, u64, ModelRc<MainListModel>);
 
 // TODO optimize this, not sure if it is possible to not copy entire model to just select item
 // https://github.com/slint-ui/slint/discussions/4595
-pub fn connect_select(app: &MainWindow) {
+pub(crate) fn connect_select(app: &MainWindow) {
     let a = app.as_weak();
     app.global::<Callabler>().on_select_items(move |select_mode| {
         let app = a.upgrade().expect("Failed to upgrade app :(");
         let active_tab = app.global::<GuiState>().get_active_tab();
-        let current_model = get_tool_model(&app, active_tab);
+        let current_model = active_tab.get_tool_model(&app);
 
-        let new_model = match select_mode {
+        let (checked_items, unchecked_items, new_model) = match select_mode {
             SelectMode::SelectAll => select_all(&current_model),
             SelectMode::UnselectAll => deselect_all(&current_model),
             SelectMode::InvertSelection => invert_selection(&current_model),
@@ -25,11 +27,12 @@ pub fn connect_select(app: &MainWindow) {
             SelectMode::SelectNewest => select_by_size_date(&current_model, active_tab, true, false),
             SelectMode::SelectOldest => select_by_size_date(&current_model, active_tab, false, false),
         };
-        set_tool_model(&app, active_tab, new_model);
+        active_tab.set_tool_model(&app, new_model);
+        change_number_of_enabled_items(&app, active_tab, checked_items as i64 - unchecked_items as i64);
     });
 }
 
-pub fn connect_showing_proper_select_buttons(app: &MainWindow) {
+pub(crate) fn connect_showing_proper_select_buttons(app: &MainWindow) {
     set_select_buttons(app);
     let a = app.as_weak();
     app.global::<Callabler>().on_tab_changed(move || {
@@ -43,13 +46,13 @@ fn set_select_buttons(app: &MainWindow) {
     let mut base_buttons = vec![SelectMode::SelectAll, SelectMode::UnselectAll, SelectMode::InvertSelection];
 
     let additional_buttons = match active_tab {
-        CurrentTab::DuplicateFiles | CurrentTab::SimilarVideos | CurrentTab::SimilarMusic => vec![
+        ActiveTab::DuplicateFiles | ActiveTab::SimilarVideos | ActiveTab::SimilarMusic => vec![
             SelectMode::SelectOldest,
             SelectMode::SelectNewest,
             SelectMode::SelectTheSmallestSize,
             SelectMode::SelectTheBiggestSize,
         ],
-        CurrentTab::SimilarImages => vec![
+        ActiveTab::SimilarImages => vec![
             SelectMode::SelectOldest,
             SelectMode::SelectNewest,
             SelectMode::SelectTheSmallestSize,
@@ -57,14 +60,14 @@ fn set_select_buttons(app: &MainWindow) {
             SelectMode::SelectTheSmallestResolution,
             SelectMode::SelectTheBiggestResolution,
         ],
-        CurrentTab::EmptyFolders
-        | CurrentTab::BigFiles
-        | CurrentTab::EmptyFiles
-        | CurrentTab::TemporaryFiles
-        | CurrentTab::InvalidSymlinks
-        | CurrentTab::BrokenFiles
-        | CurrentTab::BadExtensions => vec![],
-        CurrentTab::Settings | CurrentTab::About => vec![], // Not available in settings and about, so may be set any value here
+        ActiveTab::EmptyFolders
+        | ActiveTab::BigFiles
+        | ActiveTab::EmptyFiles
+        | ActiveTab::TemporaryFiles
+        | ActiveTab::InvalidSymlinks
+        | ActiveTab::BrokenFiles
+        | ActiveTab::BadExtensions => vec![],
+        ActiveTab::Settings | ActiveTab::About => vec![], // Not available in settings and about, so may be set any value here
     };
 
     base_buttons.extend(additional_buttons);
@@ -73,7 +76,7 @@ fn set_select_buttons(app: &MainWindow) {
     let new_select_model = base_buttons
         .into_iter()
         .map(|e| SelectModel {
-            name: translate_select_mode(e).into(),
+            name: translate_select_mode(e),
             data: e,
         })
         .collect::<Vec<_>>();
@@ -81,29 +84,17 @@ fn set_select_buttons(app: &MainWindow) {
     app.global::<GuiState>().set_select_results_list(ModelRc::new(VecModel::from(new_select_model)));
 }
 
-fn translate_select_mode(select_mode: SelectMode) -> String {
-    match select_mode {
-        SelectMode::SelectAll => "Select all".into(),
-        SelectMode::UnselectAll => "Unselect all".into(),
-        SelectMode::InvertSelection => "Invert selection".into(),
-        SelectMode::SelectTheBiggestSize => "Select the biggest size".into(),
-        SelectMode::SelectTheBiggestResolution => "Select the biggest resolution".into(),
-        SelectMode::SelectTheSmallestSize => "Select the smallest size".into(),
-        SelectMode::SelectTheSmallestResolution => "Select the smallest resolution".into(),
-        SelectMode::SelectNewest => "Select newest".into(),
-        SelectMode::SelectOldest => "Select oldest".into(),
-    }
-}
-
 // TODO, when model will be able to contain i64 instead two i32, this function could be merged with select_by_size_date
-fn select_by_resolution(model: &ModelRc<MainListModel>, active_tab: CurrentTab, biggest: bool) -> ModelRc<MainListModel> {
-    let is_header_mode = get_is_header_mode(active_tab);
+fn select_by_resolution(model: &ModelRc<MainListModel>, active_tab: ActiveTab, biggest: bool) -> SelectionResult {
+    let mut checked_items = 0;
+
+    let is_header_mode = active_tab.get_is_header_mode();
     assert!(is_header_mode); // non header modes not really have reason to use this function
 
     let mut old_data = model.iter().collect::<Vec<_>>();
     let headers_idx = find_header_idx_and_deselect_all(&mut old_data);
-    let width_idx = get_int_width_idx(active_tab);
-    let height_idx = get_int_height_idx(active_tab);
+    let width_idx = active_tab.get_int_width_idx();
+    let height_idx = active_tab.get_int_height_idx();
 
     if biggest {
         for i in 0..(headers_idx.len() - 1) {
@@ -118,6 +109,9 @@ fn select_by_resolution(model: &ModelRc<MainListModel>, active_tab: CurrentTab,
                     max_item_idx = j;
                 }
             }
+            if !old_data[max_item_idx].checked {
+                checked_items += 1;
+            }
             old_data[max_item_idx].checked = true;
         }
     } else {
@@ -133,23 +127,28 @@ fn select_by_resolution(model: &ModelRc<MainListModel>, active_tab: CurrentTab,
                     min_item_idx = j;
                 }
             }
+            if !old_data[min_item_idx].checked {
+                checked_items += 1;
+            }
             old_data[min_item_idx].checked = true;
         }
     }
 
-    ModelRc::new(VecModel::from(old_data))
+    (checked_items, 0, ModelRc::new(VecModel::from(old_data)))
 }
 
-fn select_by_size_date(model: &ModelRc<MainListModel>, active_tab: CurrentTab, biggest_newest: bool, size: bool) -> ModelRc<MainListModel> {
-    let is_header_mode = get_is_header_mode(active_tab);
+fn select_by_size_date(model: &ModelRc<MainListModel>, active_tab: ActiveTab, biggest_newest: bool, size: bool) -> SelectionResult {
+    let mut checked_items = 0;
+
+    let is_header_mode = active_tab.get_is_header_mode();
     assert!(is_header_mode); // non header modes not really have reason to use this function
 
     let mut old_data = model.iter().collect::<Vec<_>>();
     let headers_idx = find_header_idx_and_deselect_all(&mut old_data);
     let item_idx = if size {
-        get_int_size_idx(active_tab)
+        active_tab.get_int_size_idx()
     } else {
-        get_int_modification_date_idx(active_tab)
+        active_tab.get_int_modification_date_idx()
     };
 
     if biggest_newest {
@@ -165,6 +164,9 @@ fn select_by_size_date(model: &ModelRc<MainListModel>, active_tab: CurrentTab, b
                     max_item_idx = j;
                 }
             }
+            if !old_data[max_item_idx].checked {
+                checked_items += 1;
+            }
             old_data[max_item_idx].checked = true;
         }
     } else {
@@ -180,37 +182,58 @@ fn select_by_size_date(model: &ModelRc<MainListModel>, active_tab: CurrentTab, b
                     min_item_idx = j;
                 }
             }
+            if !old_data[min_item_idx].checked {
+                checked_items += 1;
+            }
             old_data[min_item_idx].checked = true;
         }
     }
 
-    ModelRc::new(VecModel::from(old_data))
+    (checked_items, 0, ModelRc::new(VecModel::from(old_data)))
 }
 
-fn select_all(model: &ModelRc<MainListModel>) -> ModelRc<MainListModel> {
+fn select_all(model: &ModelRc<MainListModel>) -> SelectionResult {
+    let mut checked_items = 0;
     let mut old_data = model.iter().collect::<Vec<_>>();
     for x in &mut old_data {
         if !x.header_row {
+            if !x.checked {
+                checked_items += 1;
+            }
             x.checked = true;
         }
     }
-    ModelRc::new(VecModel::from(old_data))
+    (checked_items, 0, ModelRc::new(VecModel::from(old_data)))
 }
 
-fn deselect_all(model: &ModelRc<MainListModel>) -> ModelRc<MainListModel> {
+fn deselect_all(model: &ModelRc<MainListModel>) -> SelectionResult {
+    let mut unchecked_items = 0;
     let mut old_data = model.iter().collect::<Vec<_>>();
-    old_data.iter_mut().for_each(|x| x.checked = false);
-    ModelRc::new(VecModel::from(old_data))
+    for x in &mut old_data {
+        if x.checked {
+            unchecked_items += 1;
+        }
+        x.checked = false;
+    }
+    (0, unchecked_items, ModelRc::new(VecModel::from(old_data)))
 }
 
-fn invert_selection(model: &ModelRc<MainListModel>) -> ModelRc<MainListModel> {
+fn invert_selection(model: &ModelRc<MainListModel>) -> SelectionResult {
+    let mut checked_items = 0;
+    let mut unchecked_items = 0;
     let mut old_data = model.iter().collect::<Vec<_>>();
     for x in &mut old_data {
         if !x.header_row {
+            if x.checked {
+                unchecked_items += 1;
+            } else {
+                checked_items += 1;
+            }
+
             x.checked = !x.checked;
         }
     }
-    ModelRc::new(VecModel::from(old_data))
+    (checked_items, unchecked_items, ModelRc::new(VecModel::from(old_data)))
 }
 
 fn find_header_idx_and_deselect_all(old_data: &mut [MainListModel]) -> Vec<usize> {
@@ -228,3 +251,88 @@ fn find_header_idx_and_deselect_all(old_data: &mut [MainListModel]) -> Vec<usize
     }
     header_idx
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::test_common::{create_model_from_model_vec, get_model_vec};
+
+    #[test]
+    fn find_header_idx_returns_correct_indices_for_headers() {
+        let mut model = get_model_vec(5);
+        model[1].header_row = true;
+        model[3].header_row = true;
+
+        let header_indices = find_header_idx_and_deselect_all(&mut model);
+
+        assert_eq!(header_indices, vec![1, 3, 5]);
+    }
+
+    #[test]
+    fn find_header_idx_marks_all_non_header_rows_as_unchecked() {
+        let mut model = get_model_vec(5);
+        model.iter_mut().for_each(|row| row.checked = true);
+        model[1].header_row = true;
+
+        find_header_idx_and_deselect_all(&mut model);
+
+        assert!(!model[0].checked);
+        assert!(model[1].checked); // header row
+        assert!(!model[2].checked);
+        assert!(!model[3].checked);
+        assert!(!model[4].checked);
+    }
+
+    #[test]
+    fn select_all_marks_all_non_header_rows_as_checked() {
+        let mut model = get_model_vec(5);
+        model[1].header_row = true;
+        let model = create_model_from_model_vec(&model);
+
+        let (checked_items, unchecked_items, new_model) = select_all(&model);
+
+        assert_eq!(checked_items, 4);
+        assert_eq!(unchecked_items, 0);
+        assert!(new_model.row_data(0).unwrap().checked);
+        assert!(!new_model.row_data(1).unwrap().checked); // header row
+        assert!(new_model.row_data(2).unwrap().checked);
+        assert!(new_model.row_data(3).unwrap().checked);
+        assert!(new_model.row_data(4).unwrap().checked);
+    }
+
+    #[test]
+    fn deselect_all_unmarks_all_rows_as_checked() {
+        let mut model = get_model_vec(5);
+        model.iter_mut().for_each(|row| row.checked = true);
+        let model = create_model_from_model_vec(&model);
+
+        let (checked_items, unchecked_items, new_model) = deselect_all(&model);
+
+        assert_eq!(checked_items, 0);
+        assert_eq!(unchecked_items, 5);
+        assert!(!new_model.row_data(0).unwrap().checked);
+        assert!(!new_model.row_data(1).unwrap().checked);
+        assert!(!new_model.row_data(2).unwrap().checked);
+        assert!(!new_model.row_data(3).unwrap().checked);
+        assert!(!new_model.row_data(4).unwrap().checked);
+    }
+
+    #[test]
+    fn invert_selection_toggles_checked_state_for_non_header_rows() {
+        let mut model = get_model_vec(5);
+        model[0].checked = true;
+        model[1].header_row = true;
+        model[2].checked = false;
+        let model = create_model_from_model_vec(&model);
+
+        let (checked_items, unchecked_items, new_model) = invert_selection(&model);
+
+        assert_eq!(checked_items, 3);
+        assert_eq!(unchecked_items, 1);
+        assert!(!new_model.row_data(0).unwrap().checked);
+        assert!(!new_model.row_data(1).unwrap().checked); // header row
+        assert!(new_model.row_data(2).unwrap().checked);
+        assert!(new_model.row_data(3).unwrap().checked);
+        assert!(new_model.row_data(4).unwrap().checked);
+    }
+}
